@@ -1,12 +1,12 @@
+#include "buffer.h"
+#include "bitset.h"
+
 #include <limits.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
 #include <assert.h>
 
-#include "bitset.h"
-#include "buffer.h"
-
 #define BITSET_BITS \
 	( CHAR_BIT * sizeof(size_t) )
 
@@ -23,12 +23,12 @@ bitset *bitset_init(size_t nbits) {
 	bitset *set;
 
 	set = malloc(sizeof(*set));
-	assert(set);
+	force_assert(set);
 
 	set->bits = calloc(BITSET_USED(nbits), sizeof(*set->bits));
 	set->nbits = nbits;
 
-	assert(set->bits);
+	force_assert(set->bits);
 
 	return set;
 }
