@@ -7,6 +7,11 @@
 #include <assert.h>
 #include <ctype.h>
 
+#if defined HAVE_STDINT_H
+# include <stdint.h>
+#elif defined HAVE_INTTYPES_H
+# include <inttypes.h>
+#endif
 
 static const char hex_chars[] = "0123456789abcdef";
 
@@ -20,7 +25,7 @@ buffer* buffer_init(void) {
 	buffer *b;
 
 	b = malloc(sizeof(*b));
-	assert(b);
+	force_assert(b);
 
 	b->ptr = NULL;
 	b->size = 0;
@@ -85,7 +90,7 @@ int buffer_prepare_copy(buffer *b, size_t size) {
 		b->size += BUFFER_PIECE_SIZE - (b->size % BUFFER_PIECE_SIZE);
 
 		b->ptr = malloc(b->size);
-		assert(b->ptr);
+		force_assert(b->ptr);
 	}
 	b->used = 0;
 	return 0;
@@ -109,7 +114,7 @@ int buffer_prepare_append(buffer *b, size_t size) {
 
 		b->ptr = malloc(b->size);
 		b->used = 0;
-		assert(b->ptr);
+		force_assert(b->ptr);
 	} else if (b->used + size > b->size) {
 		b->size += size;
 
@@ -117,7 +122,7 @@ int buffer_prepare_append(buffer *b, size_t size) {
 		b->size += BUFFER_PIECE_SIZE - (b->size % BUFFER_PIECE_SIZE);
 
 		b->ptr = realloc(b->ptr, b->size);
-		assert(b->ptr);
+		force_assert(b->ptr);
 	}
 	return 0;
 }
@@ -414,7 +419,7 @@ buffer_array* buffer_array_init(void) {
 
 	b = malloc(sizeof(*b));
 
-	assert(b);
+	force_assert(b);
 	b->ptr = NULL;
 	b->size = 0;
 	b->used = 0;
@@ -458,14 +463,14 @@ buffer *buffer_array_append_get_buffer(buffer_array *b) {
 	if (b->size == 0) {
 		b->size = 16;
 		b->ptr = malloc(sizeof(*b->ptr) * b->size);
-		assert(b->ptr);
+		force_assert(b->ptr);
 		for (i = 0; i < b->size; i++) {
 			b->ptr[i] = NULL;
 		}
 	} else if (b->size == b->used) {
 		b->size += 16;
 		b->ptr = realloc(b->ptr, sizeof(*b->ptr) * b->size);
-		assert(b->ptr);
+		force_assert(b->ptr);
 		for (i = b->used; i < b->size; i++) {
 			b->ptr[i] = NULL;
 		}
@@ -533,59 +538,37 @@ int buffer_is_equal_string(buffer *a, const char *s, size_t b_len) {
 	return buffer_is_equal(a, &b);
 }
 
-/* simple-assumption:
- *
- * most parts are equal and doing a case conversion needs time
- *
- */
-int buffer_caseless_compare(const char *a, size_t a_len, const char *b, size_t b_len) {
-	size_t ndx = 0, max_ndx;
-	size_t *al, *bl;
-	size_t mask = sizeof(*al) - 1;
+/* buffer_is_equal_caseless_string(b, CONST_STR_LEN("value")) */
+int buffer_is_equal_caseless_string(buffer *a, const char *s, size_t b_len) {
+	if (a->used != b_len + 1) return 0;
 
-	al = (size_t *)a;
-	bl = (size_t *)b;
-
-	/* is the alignment correct ? */
-	if ( ((size_t)al & mask) == 0 &&
-	     ((size_t)bl & mask) == 0 ) {
-
-		max_ndx = ((a_len < b_len) ? a_len : b_len) & ~mask;
-
-		for (; ndx < max_ndx; ndx += sizeof(*al)) {
-			if (*al != *bl) break;
-			al++; bl++;
-
-		}
-
-	}
-
-	a = (char *)al;
-	b = (char *)bl;
-
-	max_ndx = ((a_len < b_len) ? a_len : b_len);
-
-	for (; ndx < max_ndx; ndx++) {
-		int a1 = *a++, b1 = *b++;
-
-		if (a1 != b1) {
-			/* always lowercase for transitive results */
-			if (a1 >= 'A' && a1 <= 'Z') a1 |= 32;
-			if (b1 >= 'A' && b1 <= 'Z') b1 |= 32;
-
-			if ((a1 - b1) != 0) return (a1 - b1);
-		}
-	}
-
-	/* all chars are the same, and the length match too
-	 *
-	 * they are the same */
-	if (a_len == b_len) return 0;
-
-	/* if a is shorter then b, then b is larger */
-	return (a_len - b_len);
+	return (0 == strcasecmp(a->ptr, s));
 }
 
+int buffer_caseless_compare(const char *a, size_t a_len, const char *b, size_t b_len) {
+	size_t const len = (a_len < b_len) ? a_len : b_len;
+	size_t i;
+
+	for (i = 0; i < len; ++i) {
+		unsigned char ca = a[i], cb = b[i];
+		if (ca == cb) continue;
+
+		/* always lowercase for transitive results */
+#if 1
+		if (ca >= 'A' && ca <= 'Z') ca |= 32;
+		if (cb >= 'A' && cb <= 'Z') cb |= 32;
+#else
+		/* try to produce code without branching (jumps) */
+		ca |= ((unsigned char)(ca - (unsigned char)'A') <= (unsigned char)('Z' - 'A')) ? 32 : 0;
+		cb |= ((unsigned char)(cb - (unsigned char)'A') <= (unsigned char)('Z' - 'A')) ? 32 : 0;
+#endif
+
+		if (ca == cb) continue;
+		return ca - cb;
+	}
+	if (a_len == b_len) return 0;
+	return a_len - b_len;
+}
 
 /**
  * check if the rightmost bytes of the string are equal.
@@ -601,7 +584,7 @@ int buffer_is_equal_right_len(buffer *b1, buffer *b2, size_t len) {
 	if (b1->used == 0 || b2->used == 0) return 0;
 
 	/* buffers too small -> not equal */
-	if (b1->used - 1 < len || b1->used - 1 < len) return 0;
+	if (b1->used - 1 < len || b2->used - 1 < len) return 0;
 
 	if (0 == strncmp(b1->ptr + b1->used - 1 - len,
 			 b2->ptr + b2->used - 1 - len, len)) {
@@ -629,7 +612,7 @@ int buffer_copy_string_hex(buffer *b, const char *in, size_t in_len) {
 }
 
 /* everything except: ! ( ) * - . 0-9 A-Z _ a-z */
-const char encoded_chars_rel_uri_part[] = {
+static const char encoded_chars_rel_uri_part[] = {
 	/*
 	0  1  2  3  4  5  6  7  8  9  A  B  C  D  E  F
 	*/
@@ -652,7 +635,7 @@ const char encoded_chars_rel_uri_part[] = {
 };
 
 /* everything except: ! ( ) * - . / 0-9 A-Z _ a-z */
-const char encoded_chars_rel_uri[] = {
+static const char encoded_chars_rel_uri[] = {
 	/*
 	0  1  2  3  4  5  6  7  8  9  A  B  C  D  E  F
 	*/
@@ -674,7 +657,7 @@ const char encoded_chars_rel_uri[] = {
 	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  /*  F0 -  FF */
 };
 
-const char encoded_chars_html[] = {
+static const char encoded_chars_html[] = {
 	/*
 	0  1  2  3  4  5  6  7  8  9  A  B  C  D  E  F
 	*/
@@ -696,7 +679,7 @@ const char encoded_chars_html[] = {
 	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  /*  F0 -  FF */
 };
 
-const char encoded_chars_minimal_xml[] = {
+static const char encoded_chars_minimal_xml[] = {
 	/*
 	0  1  2  3  4  5  6  7  8  9  A  B  C  D  E  F
 	*/
@@ -718,7 +701,7 @@ const char encoded_chars_minimal_xml[] = {
 	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /*  F0 -  FF */
 };
 
-const char encoded_chars_hex[] = {
+static const char encoded_chars_hex[] = {
 	/*
 	0  1  2  3  4  5  6  7  8  9  A  B  C  D  E  F
 	*/
@@ -740,7 +723,7 @@ const char encoded_chars_hex[] = {
 	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  /*  F0 -  FF */
 };
 
-const char encoded_chars_http_header[] = {
+static const char encoded_chars_http_header[] = {
 	/*
 	0  1  2  3  4  5  6  7  8  9  A  B  C  D  E  F
 	*/
@@ -800,7 +783,7 @@ int buffer_append_string_encoded(buffer *b, const char *s, size_t s_len, buffer_
 		break;
 	}
 
-	assert(map != NULL);
+	force_assert(map != NULL);
 
 	/* count to-be-encoded-characters */
 	for (ds = (unsigned char *)s, d_len = 0, ndx = 0; ndx < s_len; ds++, ndx++) {
@@ -1075,3 +1058,10 @@ int buffer_to_upper(buffer *b) {
 
 	return 0;
 }
+
+void log_failed_assert(const char *filename, unsigned int line, const char *msg) {
+	/* can't use buffer here; could lead to recursive assertions */
+	fprintf(stderr, "%s.%d: %s\n", filename, line, msg);
+	fflush(stderr);
+	abort();
+}
