@@ -107,6 +107,7 @@ static int config_insert(server *srv) {
 		{ "ssl.ec-curve",                NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_SERVER },      /* 64 */
 		{ "ssl.disable-client-renegotiation", NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_SERVER },/* 65 */
 		{ "ssl.honor-cipher-order",      NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_SERVER },     /* 66 */
+		{ "ssl.empty-fragments",         NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_SERVER },     /* 67 */
 
 		{ "server.host",                 "use server.bind instead", T_CONFIG_DEPRECATED, T_CONFIG_SCOPE_UNSET },
 		{ "server.docroot",              "use server.document-root instead", T_CONFIG_DEPRECATED, T_CONFIG_SCOPE_UNSET },
@@ -154,13 +155,13 @@ static int config_insert(server *srv) {
 
 	srv->config_storage = calloc(1, srv->config_context->used * sizeof(specific_config *));
 
-	assert(srv->config_storage);
+	force_assert(srv->config_storage);
 
 	for (i = 0; i < srv->config_context->used; i++) {
 		specific_config *s;
 
 		s = calloc(1, sizeof(specific_config));
-		assert(s);
+		force_assert(s);
 		s->document_root = buffer_init();
 		s->mimetypes     = array_init();
 		s->server_name   = buffer_init();
@@ -177,8 +178,9 @@ static int config_insert(server *srv) {
 		s->max_read_idle = 60;
 		s->max_write_idle = 360;
 		s->use_xattr     = 0;
-		s->is_ssl        = 0;
+		s->ssl_enabled   = 0;
 		s->ssl_honor_cipher_order = 1;
+		s->ssl_empty_fragments = 0;
 		s->ssl_use_sslv2 = 0;
 		s->ssl_use_sslv3 = 1;
 		s->use_ipv6      = 0;
@@ -231,7 +233,7 @@ static int config_insert(server *srv) {
 		cv[27].destination = &(s->use_xattr);
 		cv[28].destination = s->mimetypes;
 		cv[29].destination = s->ssl_pemfile;
-		cv[30].destination = &(s->is_ssl);
+		cv[30].destination = &(s->ssl_enabled);
 
 		cv[31].destination = &(s->log_file_not_found);
 		cv[32].destination = &(s->log_request_handling);
@@ -250,6 +252,7 @@ static int config_insert(server *srv) {
 		cv[63].destination = s->ssl_dh_file;
 		cv[64].destination = s->ssl_ec_curve;
 		cv[66].destination = &(s->ssl_honor_cipher_order);
+		cv[67].destination = &(s->ssl_empty_fragments);
 
 		cv[49].destination = &(s->etag_use_inode);
 		cv[50].destination = &(s->etag_use_mtime);
@@ -332,17 +335,22 @@ int config_setup_connection(server *srv, connection *con) {
 
 	PATCH(range_requests);
 	PATCH(force_lowercase_filenames);
-	PATCH(is_ssl);
+	PATCH(ssl_enabled);
 
 	PATCH(ssl_pemfile);
 #ifdef USE_OPENSSL
-	PATCH(ssl_ctx);
+	PATCH(ssl_pemfile_x509);
+	PATCH(ssl_pemfile_pkey);
 #endif
 	PATCH(ssl_ca_file);
+#ifdef USE_OPENSSL
+	PATCH(ssl_ca_file_cert_names);
+#endif
 	PATCH(ssl_cipher_list);
 	PATCH(ssl_dh_file);
 	PATCH(ssl_ec_curve);
 	PATCH(ssl_honor_cipher_order);
+	PATCH(ssl_empty_fragments);
 	PATCH(ssl_use_sslv2);
 	PATCH(ssl_use_sslv3);
 	PATCH(etag_use_inode);
@@ -405,12 +413,18 @@ int config_patch_connection(server *srv, connection *con, comp_key_t comp) {
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("ssl.pemfile"))) {
 				PATCH(ssl_pemfile);
 #ifdef USE_OPENSSL
-				PATCH(ssl_ctx);
+				PATCH(ssl_pemfile_x509);
+				PATCH(ssl_pemfile_pkey);
 #endif
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("ssl.ca-file"))) {
 				PATCH(ssl_ca_file);
+#ifdef USE_OPENSSL
+				PATCH(ssl_ca_file_cert_names);
+#endif
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("ssl.honor-cipher-order"))) {
 				PATCH(ssl_honor_cipher_order);
+			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("ssl.empty-fragments"))) {
+				PATCH(ssl_empty_fragments);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("ssl.use-sslv2"))) {
 				PATCH(ssl_use_sslv2);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("ssl.use-sslv3"))) {
@@ -418,7 +432,7 @@ int config_patch_connection(server *srv, connection *con, comp_key_t comp) {
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("ssl.cipher-list"))) {
 				PATCH(ssl_cipher_list);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("ssl.engine"))) {
-				PATCH(is_ssl);
+				PATCH(ssl_enabled);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("ssl.dh-file"))) {
 				PATCH(ssl_dh_file);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("ssl.ec-curve"))) {
@@ -535,7 +549,7 @@ static int tokenizer_close(server *srv, tokenizer_t *t) {
 #endif
 static int config_skip_newline(tokenizer_t *t) {
 	int skipped = 1;
-	assert(t->input[t->offset] == '\r' || t->input[t->offset] == '\n');
+	force_assert(t->input[t->offset] == '\r' || t->input[t->offset] == '\n');
 	if (t->input[t->offset] == '\r' && t->input[t->offset + 1] == '\n') {
 		skipped ++;
 		t->offset ++;
@@ -546,7 +560,7 @@ static int config_skip_newline(tokenizer_t *t) {
 
 static int config_skip_comment(tokenizer_t *t) {
 	int i;
-	assert(t->input[t->offset] == '#');
+	force_assert(t->input[t->offset] == '#');
 	for (i = 1; t->input[t->offset + i] &&
 	     (t->input[t->offset + i] != '\n' && t->input[t->offset + i] != '\r');
 	     i++);
@@ -1000,7 +1014,7 @@ int config_parse_file(server *srv, config_t *context, const char *fn) {
 	return ret;
 }
 
-static char* getCWD() {
+static char* getCWD(void) {
 	char *s, *s1;
 	size_t len;
 #ifdef PATH_MAX
@@ -1012,7 +1026,10 @@ static char* getCWD() {
 	s = malloc(len);
 	if (!s) return NULL;
 	while (NULL == getcwd(s, len)) {
-		if (errno != ERANGE || SSIZE_MAX - len < len) return NULL;
+		if (errno != ERANGE || SSIZE_MAX - len < len) {
+			free(s);
+			return NULL;
+		}
 		len *= 2;
 		s1 = realloc(s, len);
 		if (!s1) {
@@ -1098,7 +1115,7 @@ int config_read(server *srv, const char *fn) {
 	dc = data_config_init();
 	buffer_copy_string_len(dc->key, CONST_STR_LEN("global"));
 
-	assert(context.all_configs->used == 0);
+	force_assert(context.all_configs->used == 0);
 	dc->context_ndx = context.all_configs->used;
 	array_insert_unique(context.all_configs, (data_unset *)dc);
 	context.current = dc;
@@ -1116,12 +1133,14 @@ int config_read(server *srv, const char *fn) {
 		dcwd->value->used = strlen(dcwd->value->ptr) + 1;
 		buffer_copy_string_len(dcwd->key, CONST_STR_LEN("var.CWD"));
 		array_insert_unique(srv->config, (data_unset *)dcwd);
+	} else {
+		dcwd->free((data_unset*) dcwd);
 	}
 
 	ret = config_parse_file(srv, &context, fn);
 
 	/* remains nothing if parser is ok */
-	assert(!(0 == ret && context.ok && 0 != context.configs_stack->used));
+	force_assert(!(0 == ret && context.ok && 0 != context.configs_stack->used));
 	context_free(&context);
 
 	if (0 != ret) {
@@ -1153,6 +1172,7 @@ int config_read(server *srv, const char *fn) {
 		}
 
 		prepends = (data_array *)configparser_merge_data((data_unset *)prepends, (data_unset *)modules);
+		force_assert(NULL != prepends);
 		buffer_copy_string_buffer(prepends->key, modules->key);
 		array_replace(srv->config, (data_unset *)prepends);
 		modules->free((data_unset *)modules);
@@ -1235,6 +1255,18 @@ int config_set_defaults(server *srv) {
 		{ FDEVENT_HANDLER_UNSET,          NULL }
 	};
 
+	if (!buffer_is_empty(srv->srvconf.changeroot)) {
+		if (-1 == stat(srv->srvconf.changeroot->ptr, &st1)) {
+			log_error_write(srv, __FILE__, __LINE__, "sb",
+					"server.chroot doesn't exist:", srv->srvconf.changeroot);
+			return -1;
+		}
+		if (!S_ISDIR(st1.st_mode)) {
+			log_error_write(srv, __FILE__, __LINE__, "sb",
+					"server.chroot isn't a directory:", srv->srvconf.changeroot);
+			return -1;
+		}
+	}
 
 	if (buffer_is_empty(s->document_root)) {
 		log_error_write(srv, __FILE__, __LINE__, "s",
@@ -1243,27 +1275,6 @@ int config_set_defaults(server *srv) {
 		return -1;
 	}
 
-	if (buffer_is_empty(srv->srvconf.changeroot)) {
-		if (-1 == stat(s->document_root->ptr, &st1)) {
-			log_error_write(srv, __FILE__, __LINE__, "sb",
-					"base-docroot doesn't exist:",
-					s->document_root);
-			return -1;
-		}
-
-	} else {
-		buffer_copy_string_buffer(srv->tmp_buf, srv->srvconf.changeroot);
-		buffer_append_string_buffer(srv->tmp_buf, s->document_root);
-
-		if (-1 == stat(srv->tmp_buf->ptr, &st1)) {
-			log_error_write(srv, __FILE__, __LINE__, "sb",
-					"base-docroot doesn't exist:",
-					srv->tmp_buf);
-			return -1;
-		}
-
-	}
-
 	buffer_copy_string_buffer(srv->tmp_buf, s->document_root);
 
 	buffer_to_lower(srv->tmp_buf);
@@ -1306,7 +1317,7 @@ int config_set_defaults(server *srv) {
 	}
 
 	if (srv->srvconf.port == 0) {
-		srv->srvconf.port = s->is_ssl ? 443 : 80;
+		srv->srvconf.port = s->ssl_enabled ? 443 : 80;
 	}
 
 	if (srv->srvconf.event_handler->used == 0) {
@@ -1344,7 +1355,7 @@ int config_set_defaults(server *srv) {
 		}
 	}
 
-	if (s->is_ssl) {
+	if (s->ssl_enabled) {
 		if (buffer_is_empty(s->ssl_pemfile)) {
 			/* PEM file is require */
 
