@@ -200,7 +200,7 @@ static int connection_handle_read_ssl(server *srv, connection *con) {
 	int r, ssl_err, len, count = 0, read_offset, toread;
 	buffer *b = NULL;
 
-	if (!con->conf.is_ssl) return -1;
+	if (!con->srv_socket->is_ssl) return -1;
 
 	ERR_clear_error();
 	do {
@@ -223,6 +223,12 @@ static int connection_handle_read_ssl(server *srv, connection *con) {
 
 		len = SSL_read(con->ssl, b->ptr + read_offset, toread);
 
+		if (con->renegotiations > 1 && con->conf.ssl_disable_client_renegotiation) {
+			log_error_write(srv, __FILE__, __LINE__, "s", "SSL: renegotiation initiated by client, killing connection");
+			connection_set_state(srv, con, CON_STATE_ERROR);
+			return -1;
+		}
+
 		if (len > 0) {
 			if (b->used > 0) b->used--;
 			b->used += len;
@@ -328,7 +334,7 @@ static int connection_handle_read(server *srv, connection *con) {
 	buffer *b;
 	int toread, read_offset;
 
-	if (con->conf.is_ssl) {
+	if (con->srv_socket->is_ssl) {
 		return connection_handle_read_ssl(srv, con);
 	}
 
@@ -413,15 +419,6 @@ static int connection_handle_write_prepare(server *srv, connection *con) {
 		case HTTP_METHOD_GET:
 		case HTTP_METHOD_POST:
 		case HTTP_METHOD_HEAD:
-		case HTTP_METHOD_PUT:
-		case HTTP_METHOD_MKCOL:
-		case HTTP_METHOD_DELETE:
-		case HTTP_METHOD_COPY:
-		case HTTP_METHOD_MOVE:
-		case HTTP_METHOD_PROPFIND:
-		case HTTP_METHOD_PROPPATCH:
-		case HTTP_METHOD_LOCK:
-		case HTTP_METHOD_UNLOCK:
 			break;
 		case HTTP_METHOD_OPTIONS:
 			/*
@@ -443,15 +440,8 @@ static int connection_handle_write_prepare(server *srv, connection *con) {
 			}
 			break;
 		default:
-			switch(con->http_status) {
-			case 400: /* bad request */
-			case 414: /* overload request header */
-			case 505: /* unknown protocol */
-			case 207: /* this was webdav */
-				break;
-			default:
+			if (0 == con->http_status) {
 				con->http_status = 501;
-				break;
 			}
 			break;
 		}
@@ -617,8 +607,9 @@ static int connection_handle_write_prepare(server *srv, connection *con) {
 }
 
 static int connection_handle_write(server *srv, connection *con) {
-	switch(network_write_chunkqueue(srv, con, con->write_queue)) {
+	switch(network_write_chunkqueue(srv, con, con->write_queue, MAX_WRITE_LIMIT)) {
 	case 0:
+		con->write_request_ts = srv->cur_ts;
 		if (con->file_finished) {
 			connection_set_state(srv, con, CON_STATE_RESPONSE_END);
 			joblist_append(srv, con);
@@ -635,6 +626,7 @@ static int connection_handle_write(server *srv, connection *con) {
 		joblist_append(srv, con);
 		break;
 	case 1:
+		con->write_request_ts = srv->cur_ts;
 		con->is_writable = 0;
 
 		/* not finished yet -> WRITE */
@@ -684,7 +676,6 @@ connection *connection_init(server *srv) {
 	CLEAN(physical.etag);
 	CLEAN(parse_request);
 
-	CLEAN(authed_user);
 	CLEAN(server_name);
 	CLEAN(error_handler);
 	CLEAN(dst_addr_buf);
@@ -751,7 +742,6 @@ void connections_free(server *srv) {
 		CLEAN(physical.rel_path);
 		CLEAN(parse_request);
 
-		CLEAN(authed_user);
 		CLEAN(server_name);
 		CLEAN(error_handler);
 		CLEAN(dst_addr_buf);
@@ -825,7 +815,6 @@ int connection_reset(server *srv, connection *con) {
 
 	CLEAN(parse_request);
 
-	CLEAN(authed_user);
 	CLEAN(server_name);
 	CLEAN(error_handler);
 #if defined USE_OPENSSL && ! defined OPENSSL_NO_TLSEXT
@@ -1045,7 +1034,7 @@ found_header_end:
 
 			weWant = con->request.content_length - dst_cq->bytes_in;
 
-			assert(c->mem->used);
+			force_assert(c->mem->used);
 
 			weHave = c->mem->used - c->offset - 1;
 
@@ -1078,9 +1067,7 @@ found_header_end:
 						if (dst_c->file.fd == -1) {
 							/* this should not happen as we cache the fd, but you never know */
 							dst_c->file.fd = open(dst_c->file.name->ptr, O_WRONLY | O_APPEND);
-#ifdef FD_CLOEXEC
-							fcntl(dst_c->file.fd, F_SETFD, FD_CLOEXEC);
-#endif
+							fd_close_on_exec(dst_c->file.fd);
 						}
 					} else {
 						/* the chunk is too large now, close it */
@@ -1182,7 +1169,7 @@ static handler_t connection_handle_fdevent(server *srv, void *context, int reven
 
 	joblist_append(srv, con);
 
-	if (con->conf.is_ssl) {
+	if (con->srv_socket->is_ssl) {
 		/* ssl may read and write for both reads and writes */
 		if (revents & (FDEVENT_IN | FDEVENT_OUT)) {
 			con->is_readable = 1;
@@ -1251,8 +1238,6 @@ static handler_t connection_handle_fdevent(server *srv, void *context, int reven
 			log_error_write(srv, __FILE__, __LINE__, "ds",
 					con->fd,
 					"handle write failed.");
-		} else if (con->state == CON_STATE_WRITE) {
-			con->write_request_ts = srv->cur_ts;
 		}
 	}
 
@@ -1352,11 +1337,9 @@ connection *connection_accept(server *srv, server_socket *srv_socket) {
 				return NULL;
 			}
 
-#ifndef OPENSSL_NO_TLSEXT
+			con->renegotiations = 0;
 			SSL_set_app_data(con->ssl, con);
-#endif
 			SSL_set_accept_state(con->ssl);
-			con->conf.is_ssl=1;
 
 			if (1 != (SSL_set_fd(con->ssl, cnt))) {
 				log_error_write(srv, __FILE__, __LINE__, "ss", "SSL:",
@@ -1401,12 +1384,6 @@ int connection_state_machine(server *srv, connection *con) {
 
 			connection_set_state(srv, con, CON_STATE_READ);
 
-			/* patch con->conf.is_ssl if the connection is a ssl-socket already */
-
-#ifdef USE_OPENSSL
-			con->conf.is_ssl = srv_sock->is_ssl;
-#endif
-
 			break;
 		case CON_STATE_REQUEST_END: /* transient */
 			if (srv->srvconf.log_state_handling) {
@@ -1498,6 +1475,7 @@ int connection_state_machine(server *srv, connection *con) {
 				break;
 			case HANDLER_COMEBACK:
 				done = -1;
+				/* fallthrough */
 			case HANDLER_WAIT_FOR_EVENT:
 				/* come back here */
 				connection_set_state(srv, con, CON_STATE_HANDLE_REQUEST);
@@ -1667,8 +1645,6 @@ int connection_state_machine(server *srv, connection *con) {
 							con->fd,
 							"handle write failed.");
 					connection_set_state(srv, con, CON_STATE_ERROR);
-				} else if (con->state == CON_STATE_WRITE) {
-					con->write_request_ts = srv->cur_ts;
 				}
 			}
 
@@ -1708,11 +1684,18 @@ int connection_state_machine(server *srv, connection *con) {
 										ERR_error_string(err, NULL));
 							} while((err = ERR_get_error()));
 						} else if (errno != 0) { /* ssl bug (see lighttpd ticket #2213): sometimes errno == 0 */
-							log_error_write(srv, __FILE__, __LINE__, "sddds", "SSL (error):",
+							switch(errno) {
+							case EPIPE:
+							case ECONNRESET:
+								break;
+							default:
+								log_error_write(srv, __FILE__, __LINE__, "sddds", "SSL (error):",
 									ssl_r, ret, errno,
 									strerror(errno));
+								break;
+							}
 						}
-	
+
 						break;
 					default:
 						while((err = ERR_get_error())) {
@@ -1720,12 +1703,12 @@ int connection_state_machine(server *srv, connection *con) {
 									ssl_r, ret,
 									ERR_error_string(err, NULL));
 						}
-	
+
 						break;
 					}
 				}
+				ERR_clear_error();
 			}
-			ERR_clear_error();
 #endif
 
 			switch(con->mode) {
