@@ -8,13 +8,12 @@
  */
 int openDevNull(int fd);
 
-#define WP() log_error_write(srv, __FILE__, __LINE__, "");
-
 int open_logfile_or_pipe(server *srv, const char* logfile);
 
 int log_error_open(server *srv);
 int log_error_close(server *srv);
 int log_error_write(server *srv, const char *filename, unsigned int line, const char *fmt, ...);
+int log_error_write_multiline_buffer(server *srv, const char *filename, unsigned int line, buffer *multiline, const char *fmt, ...);
 int log_error_cycle(server *srv);
 
 #endif
