@@ -72,7 +72,7 @@ SETDEFAULTS_FUNC(mod_alias_set_defaults) {
 
 	if (!p) return HANDLER_ERROR;
 
-	p->config_storage = calloc(1, srv->config_context->used * sizeof(specific_config *));
+	p->config_storage = calloc(1, srv->config_context->used * sizeof(plugin_config *));
 
 	for (i = 0; i < srv->config_context->used; i++) {
 		plugin_config *s;
@@ -156,7 +156,8 @@ PHYSICALPATH_FUNC(mod_alias_physical_handler) {
 	mod_alias_patch_connection(srv, con, p);
 
 	/* not to include the tailing slash */
-	basedir_len = (con->physical.basedir->used - 1) - 1;
+	basedir_len = (con->physical.basedir->used - 1);
+	if ('/' == con->physical.basedir->ptr[basedir_len-1]) --basedir_len;
 	uri_len = con->physical.path->used - 1 - basedir_len;
 	uri_ptr = con->physical.path->ptr + basedir_len;
 
