@@ -82,10 +82,10 @@ typedef struct {
 	buffer *response_header;
 } handler_ctx;
 
-static handler_ctx * cgi_handler_ctx_init() {
+static handler_ctx * cgi_handler_ctx_init(void) {
 	handler_ctx *hctx = calloc(1, sizeof(*hctx));
 
-	assert(hctx);
+	force_assert(hctx);
 
 	hctx->response = buffer_init();
 	hctx->response_header = buffer_init();
@@ -107,7 +107,7 @@ INIT_FUNC(mod_cgi_init) {
 
 	p = calloc(1, sizeof(*p));
 
-	assert(p);
+	force_assert(p);
 
 	p->tmp_buf = buffer_init();
 	p->parse_response = buffer_init();
@@ -157,13 +157,13 @@ SETDEFAULTS_FUNC(mod_fastcgi_set_defaults) {
 
 	if (!p) return HANDLER_ERROR;
 
-	p->config_storage = calloc(1, srv->config_context->used * sizeof(specific_config *));
+	p->config_storage = calloc(1, srv->config_context->used * sizeof(plugin_config *));
 
 	for (i = 0; i < srv->config_context->used; i++) {
 		plugin_config *s;
 
 		s = calloc(1, sizeof(plugin_config));
-		assert(s);
+		force_assert(s);
 
 		s->cgi    = array_init();
 		s->execute_x_only = 0;
@@ -918,17 +918,10 @@ static int cgi_create_env(server *srv, connection *con, plugin_data *p, buffer *
 			);
 		cgi_env_add(&env, CONST_STR_LEN("REMOTE_PORT"), buf, strlen(buf));
 
-		if (!buffer_is_empty(con->authed_user)) {
-			cgi_env_add(&env, CONST_STR_LEN("REMOTE_USER"),
-				    CONST_BUF_LEN(con->authed_user));
+		if (buffer_is_equal_caseless_string(con->uri.scheme, CONST_STR_LEN("https"))) {
+			cgi_env_add(&env, CONST_STR_LEN("HTTPS"), CONST_STR_LEN("on"));
 		}
 
-#ifdef USE_OPENSSL
-	if (srv_sock->is_ssl) {
-		cgi_env_add(&env, CONST_STR_LEN("HTTPS"), CONST_STR_LEN("on"));
-	}
-#endif
-
 		/* request.content_length < SSIZE_MAX, see request.c */
 		LI_ltostr(buf, con->request.content_length);
 		cgi_env_add(&env, CONST_STR_LEN("CONTENT_LENGTH"), buf, strlen(buf));
@@ -1099,7 +1092,7 @@ static int cgi_create_env(server *srv, connection *con, plugin_data *p, buffer *
 
 						c->file.mmap.length = c->file.length;
 
-						if (MAP_FAILED == (c->file.mmap.start = mmap(0,  c->file.mmap.length, PROT_READ, MAP_SHARED, c->file.fd, 0))) {
+						if (MAP_FAILED == (c->file.mmap.start = mmap(NULL,  c->file.mmap.length, PROT_READ, MAP_SHARED, c->file.fd, 0))) {
 							log_error_write(srv, __FILE__, __LINE__, "ssbd", "mmap failed: ",
 									strerror(errno), c->file.name,  c->file.fd);
 
