@@ -1,7 +1,3 @@
-#include <ctype.h>
-#include <stdlib.h>
-#include <string.h>
-
 #include "base.h"
 #include "log.h"
 #include "buffer.h"
@@ -10,6 +6,10 @@
 
 #include "inet_ntop_cache.h"
 
+#include <ctype.h>
+#include <stdlib.h>
+#include <string.h>
+
 /**
  * mod_evasive
  *
@@ -27,6 +27,7 @@
 
 typedef struct {
 	unsigned short max_conns;
+	unsigned short silent;
 } plugin_config;
 
 typedef struct {
@@ -72,19 +73,22 @@ SETDEFAULTS_FUNC(mod_evasive_set_defaults) {
 	size_t i = 0;
 
 	config_values_t cv[] = {
-		{ "evasive.max-conns-per-ip",    NULL, T_CONFIG_SHORT, T_CONFIG_SCOPE_CONNECTION },
+		{ "evasive.max-conns-per-ip",    NULL, T_CONFIG_SHORT, T_CONFIG_SCOPE_CONNECTION },   /* 0 */
+		{ "evasive.silent",              NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_CONNECTION }, /* 1 */
 		{ NULL,                          NULL, T_CONFIG_UNSET, T_CONFIG_SCOPE_UNSET }
 	};
 
-	p->config_storage = calloc(1, srv->config_context->used * sizeof(specific_config *));
+	p->config_storage = calloc(1, srv->config_context->used * sizeof(plugin_config *));
 
 	for (i = 0; i < srv->config_context->used; i++) {
 		plugin_config *s;
 
 		s = calloc(1, sizeof(plugin_config));
 		s->max_conns       = 0;
+		s->silent          = 0;
 
 		cv[0].destination = &(s->max_conns);
+		cv[1].destination = &(s->silent);
 
 		p->config_storage[i] = s;
 
@@ -103,6 +107,7 @@ static int mod_evasive_patch_connection(server *srv, connection *con, plugin_dat
 	plugin_config *s = p->config_storage[0];
 
 	PATCH(max_conns);
+	PATCH(silent);
 
 	/* skip the first, the global context */
 	for (i = 1; i < srv->config_context->used; i++) {
@@ -118,6 +123,8 @@ static int mod_evasive_patch_connection(server *srv, connection *con, plugin_dat
 
 			if (buffer_is_equal_string(du->key, CONST_STR_LEN("evasive.max-conns-per-ip"))) {
 				PATCH(max_conns);
+			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("evasive.silent"))) {
+				PATCH(silent);
 			}
 		}
 	}
@@ -172,9 +179,11 @@ URIHANDLER_FUNC(mod_evasive_uri_handler) {
 		conns_by_ip++;
 
 		if (conns_by_ip > p->conf.max_conns) {
-			log_error_write(srv, __FILE__, __LINE__, "ss",
-				inet_ntop_cache_get_ip(srv, &(con->dst_addr)),
-				"turned away. Too many connections.");
+			if (!p->conf.silent) {
+				log_error_write(srv, __FILE__, __LINE__, "ss",
+					inet_ntop_cache_get_ip(srv, &(con->dst_addr)),
+					"turned away. Too many connections.");
+			}
 
 			con->http_status = 403;
 			con->mode = DIRECT;
@@ -186,6 +195,7 @@ URIHANDLER_FUNC(mod_evasive_uri_handler) {
 }
 
 
+int mod_evasive_plugin_init(plugin *p);
 int mod_evasive_plugin_init(plugin *p) {
 	p->version     = LIGHTTPD_VERSION_ID;
 	p->name        = buffer_init_string("evasive");
