@@ -1,8 +1,3 @@
-#include <ctype.h>
-#include <stdlib.h>
-#include <string.h>
-#include <time.h>
-
 #include "base.h"
 #include "log.h"
 #include "buffer.h"
@@ -11,6 +6,11 @@
 #include "plugin.h"
 #include "stat_cache.h"
 
+#include <ctype.h>
+#include <stdlib.h>
+#include <string.h>
+#include <time.h>
+
 /**
  * this is a expire module for a lighttpd
  *
@@ -62,9 +62,9 @@ FREE_FUNC(mod_expire_free) {
 		size_t i;
 		for (i = 0; i < srv->config_context->used; i++) {
 			plugin_config *s = p->config_storage[i];
+			if (!s) continue;
 
 			array_free(s->expire_url);
-
 			free(s);
 		}
 		free(p->config_storage);
@@ -75,10 +75,10 @@ FREE_FUNC(mod_expire_free) {
 	return HANDLER_GO_ON;
 }
 
-static int mod_expire_get_offset(server *srv, plugin_data *p, buffer *expire, int *offset) {
+static int mod_expire_get_offset(server *srv, plugin_data *p, buffer *expire, time_t *offset) {
 	char *ts;
 	int type = -1;
-	int retts = 0;
+	time_t retts = 0;
 
 	UNUSED(p);
 
@@ -221,7 +221,7 @@ SETDEFAULTS_FUNC(mod_expire_set_defaults) {
 
 	if (!p) return HANDLER_ERROR;
 
-	p->config_storage = calloc(1, srv->config_context->used * sizeof(specific_config *));
+	p->config_storage = calloc(1, srv->config_context->used * sizeof(plugin_config *));
 
 	for (i = 0; i < srv->config_context->used; i++) {
 		plugin_config *s;
@@ -302,35 +302,41 @@ URIHANDLER_FUNC(mod_expire_path_handler) {
 		if (ds->key->used == 0) continue;
 
 		if (0 == strncmp(con->uri.path->ptr, ds->key->ptr, ct_len)) {
-			int ts;
-			time_t t;
+			time_t ts, expires;
 			size_t len;
 			stat_cache_entry *sce = NULL;
 
-			stat_cache_get_entry(srv, con, con->physical.path, &sce);
+			/* if stat fails => sce == NULL, ignore return value */
+			(void) stat_cache_get_entry(srv, con, con->physical.path, &sce);
 
 			switch(mod_expire_get_offset(srv, p, ds->value, &ts)) {
 			case 0:
 				/* access */
-				t = (ts + srv->cur_ts);
+				expires = (ts + srv->cur_ts);
 				break;
 			case 1:
 				/* modification */
 
-				t = (ts + sce->st.st_mtime);
+				/* can't set modification based expire header if
+				 * mtime is not available
+				 */
+				if (NULL == sce) return HANDLER_GO_ON;
+
+				expires = (ts + sce->st.st_mtime);
 				break;
 			default:
 				/* -1 is handled at parse-time */
-				break;
+				return HANDLER_ERROR;
 			}
 
+			/* expires should be at least srv->cur_ts */
+			if (expires < srv->cur_ts) expires = srv->cur_ts;
 
 			if (0 == (len = strftime(p->expire_tstmp->ptr, p->expire_tstmp->size - 1,
-					   "%a, %d %b %Y %H:%M:%S GMT", gmtime(&(t))))) {
+					   "%a, %d %b %Y %H:%M:%S GMT", gmtime(&(expires))))) {
 				/* could not set expire header, out of mem */
 
 				return HANDLER_GO_ON;
-
 			}
 
 			p->expire_tstmp->used = len + 1;
@@ -339,10 +345,10 @@ URIHANDLER_FUNC(mod_expire_path_handler) {
 			response_header_overwrite(srv, con, CONST_STR_LEN("Expires"), CONST_BUF_LEN(p->expire_tstmp));
 
 			/* HTTP/1.1 */
-			buffer_copy_string(p->expire_tstmp, "max-age=");
-			buffer_append_long(p->expire_tstmp, ts);
+			buffer_copy_string_len(p->expire_tstmp, CONST_STR_LEN("max-age="));
+			buffer_append_long(p->expire_tstmp, expires - srv->cur_ts); /* as expires >= srv->cur_ts the difference is >= 0 */
 
-			response_header_overwrite(srv, con, CONST_STR_LEN("Cache-Control"), CONST_BUF_LEN(p->expire_tstmp));
+			response_header_append(srv, con, CONST_STR_LEN("Cache-Control"), CONST_BUF_LEN(p->expire_tstmp));
 
 			return HANDLER_GO_ON;
 		}
@@ -354,6 +360,7 @@ URIHANDLER_FUNC(mod_expire_path_handler) {
 
 /* this function is called at dlopen() time and inits the callbacks */
 
+int mod_expire_plugin_init(plugin *p);
 int mod_expire_plugin_init(plugin *p) {
 	p->version     = LIGHTTPD_VERSION_ID;
 	p->name        = buffer_init_string("expire");
