@@ -103,7 +103,7 @@ SETDEFAULTS_FUNC(mod_magnet_set_defaults) {
 
 	if (!p) return HANDLER_ERROR;
 
-	p->config_storage = calloc(1, srv->config_context->used * sizeof(specific_config *));
+	p->config_storage = calloc(1, srv->config_context->used * sizeof(plugin_config *));
 
 	for (i = 0; i < srv->config_context->used; i++) {
 		plugin_config *s;
@@ -170,6 +170,7 @@ static int magnet_pairs(lua_State *L) {
 		return lua_gettop(L);
 	} else {
 		lua_pushvalue(L, lua_upvalueindex(1));
+		lua_insert(L, 1);
 		lua_call(L, lua_gettop(L) - 1, LUA_MULTRET);
 		return lua_gettop(L);
 	}
@@ -445,6 +446,7 @@ typedef struct {
 		MAGNET_ENV_PHYICAL_PATH,
 		MAGNET_ENV_PHYICAL_REL_PATH,
 		MAGNET_ENV_PHYICAL_DOC_ROOT,
+		MAGNET_ENV_PHYICAL_BASEDIR,
 
 		MAGNET_ENV_URI_PATH,
 		MAGNET_ENV_URI_PATH_RAW,
@@ -465,6 +467,7 @@ static const magnet_env_t magnet_env[] = {
 	{ "physical.path", MAGNET_ENV_PHYICAL_PATH },
 	{ "physical.rel-path", MAGNET_ENV_PHYICAL_REL_PATH },
 	{ "physical.doc-root", MAGNET_ENV_PHYICAL_DOC_ROOT },
+	{ "physical.basedir", MAGNET_ENV_PHYICAL_BASEDIR },
 
 	{ "uri.path", MAGNET_ENV_URI_PATH },
 	{ "uri.path-raw", MAGNET_ENV_URI_PATH_RAW },
@@ -496,6 +499,7 @@ static buffer *magnet_env_get_buffer_by_id(server *srv, connection *con, int id)
 	case MAGNET_ENV_PHYICAL_PATH: dest = con->physical.path; break;
 	case MAGNET_ENV_PHYICAL_REL_PATH: dest = con->physical.rel_path; break;
 	case MAGNET_ENV_PHYICAL_DOC_ROOT: dest = con->physical.doc_root; break;
+	case MAGNET_ENV_PHYICAL_BASEDIR: dest = con->physical.basedir; break;
 
 	case MAGNET_ENV_URI_PATH: dest = con->uri.path; break;
 	case MAGNET_ENV_URI_PATH_RAW: dest = con->uri.path_raw; break;
@@ -692,7 +696,7 @@ static int magnet_copy_response_header(server *srv, connection *con, plugin_data
 	/* lighty.header */
 
 	lua_getfield(L, -1, "lighty"); /* lighty.* from the env  */
-	assert(lua_istable(L, -1));
+	force_assert(lua_istable(L, -1));
 
 	lua_getfield(L, -1, "header"); /* lighty.header */
 	if (lua_istable(L, -1)) {
@@ -736,11 +740,11 @@ static int magnet_attach_content(server *srv, connection *con, plugin_data *p, l
 	 * get the environment of the function
 	 */
 
-	assert(lua_isfunction(L, -1));
+	force_assert(lua_isfunction(L, -1));
 	lua_getfenv(L, -1); /* -1 is the function */
 
 	lua_getfield(L, -1, "lighty"); /* lighty.* from the env  */
-	assert(lua_istable(L, -1));
+	force_assert(lua_istable(L, -1));
 
 	lua_getfield(L, -1, "content"); /* lighty.content */
 	if (lua_istable(L, -1)) {
@@ -762,12 +766,17 @@ static int magnet_attach_content(server *srv, connection *con, plugin_data *p, l
 				lua_getfield(L, -3, "offset");
 
 				if (lua_isstring(L, -3)) { /* filename has to be a string */
-					buffer *fn = buffer_init();
+					buffer *fn;
 					stat_cache_entry *sce;
+					const char *fn_str;
+					handler_t res;
 
-					buffer_copy_string(fn, lua_tostring(L, -3));
+					fn_str = lua_tostring(L, -3);
+					fn = buffer_init_string(fn_str);
 
-					if (HANDLER_GO_ON == stat_cache_get_entry(srv, con, fn, &sce)) {
+					res = stat_cache_get_entry(srv, con, fn, &sce);
+
+					if (HANDLER_GO_ON == res) {
 						off_t off = 0;
 						off_t len = 0;
 
@@ -782,11 +791,13 @@ static int magnet_attach_content(server *srv, connection *con, plugin_data *p, l
 						}
 
 						if (off < 0) {
-							return luaL_error(L, "offset for '%s' is negative", fn->ptr);
+							buffer_free(fn);
+							return luaL_error(L, "offset for '%s' is negative", fn_str);
 						}
 
 						if (len < off) {
-							return luaL_error(L, "offset > length for '%s'", fn->ptr);
+							buffer_free(fn);
+							return luaL_error(L, "offset > length for '%s'", fn_str);
 						}
 
 						chunkqueue_append_file(con->write_queue, fn, off, len - off);
@@ -824,9 +835,36 @@ static int magnet_attach_content(server *srv, connection *con, plugin_data *p, l
 	return 0;
 }
 
+static int traceback (lua_State *L) {
+	if (!lua_isstring(L, 1))  /* 'message' not a string? */
+		return 1;  /* keep it intact */
+	lua_getfield(L, LUA_GLOBALSINDEX, "debug");
+	if (!lua_istable(L, -1)) {
+		lua_pop(L, 1);
+		return 1;
+	}
+	lua_getfield(L, -1, "traceback");
+	if (!lua_isfunction(L, -1)) {
+		lua_pop(L, 2);
+		return 1;
+	}
+	lua_pushvalue(L, 1);  /* pass error message */
+	lua_pushinteger(L, 2);  /* skip this function and traceback */
+	lua_call(L, 2, 1);  /* call debug.traceback */
+	return 1;
+}
+
+static int push_traceback(lua_State *L, int narg) {
+	int base = lua_gettop(L) - narg;  /* function index */
+	lua_pushcfunction(L, traceback);
+	lua_insert(L, base);
+	return base;
+}
+
 static handler_t magnet_attract(server *srv, connection *con, plugin_data *p, buffer *name) {
 	lua_State *L;
 	int lua_return_value = -1;
+	int errfunc;
 	/* get the script-context */
 
 
@@ -842,7 +880,7 @@ static handler_t magnet_attract(server *srv, connection *con, plugin_data *p, bu
 
 		lua_pop(L, 1);
 
-		assert(lua_gettop(L) == 0); /* only the function should be on the stack */
+		force_assert(lua_gettop(L) == 0); /* only the function should be on the stack */
 
 		con->http_status = 500;
 		con->mode = DIRECT;
@@ -955,23 +993,26 @@ static handler_t magnet_attract(server *srv, connection *con, plugin_data *p, bu
 
 	lua_setfenv(L, -2); /* on the stack should be a modified env (sp -= 1) */
 
-	if (lua_pcall(L, 0, 1, 0)) {
+	errfunc = push_traceback(L, 0);
+	if (lua_pcall(L, 0, 1, errfunc)) {
+		lua_remove(L, errfunc);
 		log_error_write(srv, __FILE__, __LINE__,
 			"ss",
 			"lua_pcall():",
 			lua_tostring(L, -1));
 		lua_pop(L, 1); /* remove the error-msg and the function copy from the stack */
 
-		assert(lua_gettop(L) == 1); /* only the function should be on the stack */
+		force_assert(lua_gettop(L) == 1); /* only the function should be on the stack */
 
 		con->http_status = 500;
 		con->mode = DIRECT;
 
 		return HANDLER_FINISHED;
 	}
+	lua_remove(L, errfunc);
 
 	/* we should have the function-copy and the return value on the stack */
-	assert(lua_gettop(L) == 2);
+	force_assert(lua_gettop(L) == 2);
 
 	if (lua_isnumber(L, -1)) {
 		/* if the ret-value is a number, take it */
@@ -997,16 +1038,16 @@ static handler_t magnet_attract(server *srv, connection *con, plugin_data *p, bu
 			con->mode = DIRECT;
 		}
 
-		assert(lua_gettop(L) == 1); /* only the function should be on the stack */
+		force_assert(lua_gettop(L) == 1); /* only the function should be on the stack */
 
 		/* we are finished */
 		return HANDLER_FINISHED;
 	} else if (MAGNET_RESTART_REQUEST == lua_return_value) {
-		assert(lua_gettop(L) == 1); /* only the function should be on the stack */
+		force_assert(lua_gettop(L) == 1); /* only the function should be on the stack */
 
 		return HANDLER_COMEBACK;
 	} else {
-		assert(lua_gettop(L) == 1); /* only the function should be on the stack */
+		force_assert(lua_gettop(L) == 1); /* only the function should be on the stack */
 
 		return HANDLER_GO_ON;
 	}
