@@ -114,7 +114,7 @@ typedef struct {
 /* ok, we need a prototype */
 static handler_t proxy_handle_fdevent(server *srv, void *ctx, int revents);
 
-static handler_ctx * handler_ctx_init() {
+static handler_ctx * handler_ctx_init(void) {
 	handler_ctx * hctx;
 
 
@@ -194,7 +194,7 @@ SETDEFAULTS_FUNC(mod_proxy_set_defaults) {
 		{ NULL,                        NULL, T_CONFIG_UNSET, T_CONFIG_SCOPE_UNSET }
 	};
 
-	p->config_storage = calloc(1, srv->config_context->used * sizeof(specific_config *));
+	p->config_storage = calloc(1, srv->config_context->used * sizeof(plugin_config *));
 
 	for (i = 0; i < srv->config_context->used; i++) {
 		plugin_config *s;
@@ -298,6 +298,7 @@ SETDEFAULTS_FUNC(mod_proxy_set_defaults) {
 					pcv[1].destination = &(df->port);
 
 					if (0 != config_insert_values_internal(srv, da_host->value, pcv)) {
+						df->free((data_unset*) df);
 						return HANDLER_ERROR;
 					}
 
@@ -309,6 +310,7 @@ SETDEFAULTS_FUNC(mod_proxy_set_defaults) {
 								da_host->key,
 								"host");
 
+						df->free((data_unset*) df);
 						return HANDLER_ERROR;
 					}
 
@@ -463,7 +465,7 @@ static int proxy_create_env(server *srv, handler_ctx *hctx) {
 	    !buffer_is_empty(con->request.http_host)) {
 		proxy_set_header(con, "X-Host", con->request.http_host->ptr);
 	}
-	proxy_set_header(con, "X-Forwarded-Proto", con->conf.is_ssl ? "https" : "http");
+	proxy_set_header(con, "X-Forwarded-Proto", con->uri.scheme->ptr);
 
 	/* request header */
 	for (i = 0; i < con->request.headers->used; i++) {
@@ -684,7 +686,7 @@ static int proxy_demux_response(server *srv, handler_ctx *hctx) {
 		}
 
 		/* this should be catched by the b > 0 above */
-		assert(r);
+		force_assert(r);
 
 		hctx->response->used += r;
 		hctx->response->ptr[hctx->response->used - 1] = '\0';
@@ -825,7 +827,7 @@ static handler_t proxy_write_request(server *srv, handler_ctx *hctx) {
 
 		/* fall through */
 	case PROXY_STATE_WRITE:;
-		ret = srv->network_backend_write(srv, con, hctx->fd, hctx->wb);
+		ret = srv->network_backend_write(srv, con, hctx->fd, hctx->wb, MAX_WRITE_LIMIT);
 
 		chunkqueue_remove_finished_chunks(hctx->wb);
 
@@ -1161,7 +1163,7 @@ static handler_t mod_proxy_check_extension(server *srv, connection *con, void *p
 				if (s_len > ct_len + 1) {
 					char *pi_offset;
 
-					if (0 != (pi_offset = strchr(fn->ptr + ct_len + 1, '/'))) {
+					if (NULL != (pi_offset = strchr(fn->ptr + ct_len + 1, '/'))) {
 						path_info_offset = pi_offset - fn->ptr;
 					}
 				}
@@ -1256,7 +1258,7 @@ static handler_t mod_proxy_check_extension(server *srv, connection *con, void *p
 		}
 
 		/* just to be sure */
-		assert(extension->value->used < INT_MAX);
+		force_assert(extension->value->used < INT_MAX);
 
 		host = (data_proxy *)extension->value->data[0];
 
